#include <stdio.h>
#include "ustring.h"

int main(void)
{
  char buf8[1024] = {
    BOM_8_1, BOM_8_2, BOM_8_3,     // UTF-8 Byte Order Mark
    '\x7E',                        // Tilde
    '\xC3', '\xB6',                // Umlaut oe
    '\xE2', '\x82', '\xAC',        // Euro Symbol
    '\xF0', '\xA4', '\xBD', '\x9C' // CJK Ideogramm
  };

// get info
  size_t len = 0U;
  size_t size = Validate8(buf8, &len);
  if (size > 0U)
    puts("The UTF-8 source is valid.");
  else
  {
    puts("The UTF-8 source is invalid.");
    return 1;
  }

  printf("It has a size of %lu bytes and contains %lu code points.\n~~~~~~~~~~~~~~\n", (unsigned long)size, (unsigned long)len);


// determine if the source has BOM
  printf("The UTF-8 source has %s byte order mark.\n~~~~~~~~~~~~~~\n", HasBOM8(buf8) == 1 ? "a" : "no");


// convert code points separately
  uint16_t buf16[2] = {0};
  char *ptr8 = buf8;
  int written = 0;

  while (*ptr8 != '\0')
  {
    printf("Source pointer: %p\n", ptr8);
    written = ConvertCodePoint((CCP_DST_T)buf16, 16, (CCP_SRC_T)&ptr8, 8);
    printf("Next pointer  : %p\n", ptr8);
    printf("UTF-16 Code units written: %d, values:", written);
    for (int i = 0; i < written; ++i)
      printf(" %04X", buf16[i]);
    puts("\n~~~~~~~~~~~~~~");
  }


// Code Page 850 encoding
  char     buf850[1024] = {0};
  uint32_t buf32[1024]  = {BOM_CPT, 0x2591U, 0x2592U, 0x2593U, 0x2588U, 0x2534U, 0x2569U, 0x00A9U};

  puts("new UTF-32 source code points");
  for (uint32_t *it = buf32; *it != 0u; printf("%08X ", *it++));

  Utf32ToCP850((HasBOM32(buf32) ? &buf32[1] : buf32), buf850, sizeof(buf850));
  puts("\n\noutput of the CP 850 encoded string");
  puts(buf850);

  Cp850ToUtf32(buf850, buf32, sizeof(buf32));
  puts("\nconverted back to UTF-32");
  for (uint32_t *it = buf32; *it != 0u; printf("%08X ", *it++));
  puts("\n~~~~~~~~~~~~~~");

  return 0;
}
